/*
  Optical Heart Rate Detection (PBA Algorithm) using the MAX30105 Breakout
  By: Nathan Seidle @ SparkFun Electronics
  Date: October 2nd, 2016
  https://github.com/sparkfun/MAX30105_Breakout

  Modified by Sarah Ornellas
  MIT, Cambridge, MA
  Date: November 2023
  Using MAX30102

  This is a demo to show the reading of heart rate or beats per minute (BPM) using
  a Penpheral Beat Amplitude (PBA) algorithm.

  It is best to attach the sensor to your finger using a rubber band or other tightening
  device. Humans are generally bad at applying constant pressure to a thing. When you
  press your finger against the sensor it varies enough to cause the blood in your
  finger to flow differently which causes the sensor readings to go wonky.

  Hardware Connections (Breakoutboard to Arduino):
  -5V = 5V (3.3V is allowed)
  -GND = GND
  -SDA = A4 (or SDA)
  -SCL = A5 (or SCL)
  -INT = Not connected

  The MAX30105 Breakout can handle 5V or 3.3V I2C logic. We recommend powering the board with 5V
  but it will also run at 3.3V.
*/

#include <Wire.h>
#include "MAX30105.h"

#include "heartRate.h"

MAX30105 particleSensor;

const byte RATE_SIZE = 4; //Increase this for more averaging of IR values. 4 is good.
byte rates[RATE_SIZE]; //Array of heart rates
byte rateSpot = 0;
long lastBeat = 0; //Time at which the last beat occurred

float beatsPerMinute;
int beatAvg;
int display_ct = 0;

const int NUM_READINGS = 10; //Adjust the number of readings before displaying Heart Rate
int valid_HR[NUM_READINGS];
int index_valid = 0;
int display_HR;

uint8_t LED1pin = 2;
bool LED1status = LOW;

uint8_t LED2pin = 5;
bool LED2status = LOW;

uint8_t BUTTONpin = 10;
bool BUTTONstate = LOW;

enum State {
  INIT,
  MEASURE,
};

State currentState = MEASURE;

int lastUpdateMillis = 0; // Variable to store the last update time
int lastnonValidMillis = 0; // Variable to store the last non valid time
int UPDATE_INTERVAL = 1000; // Update interval in milliseconds (adjust as needed)
int current_interval = 0; // Current interval between last valid reading
int noFinger_interval = 0; // Current interval between last not valid reading


// create function to check if there are 0 readings
bool hasZeroElement() {
  for (int i = 0; i < NUM_READINGS; i++) {
    if (valid_HR[i] == 0) {
      return true;
    }
  }
  return false;
}

void setup()
{
  Serial.begin(115200);
  Serial.println("Initializing...");

  pinMode(LED1pin, OUTPUT);
  pinMode(LED2pin, OUTPUT);
  pinMode(BUTTONpin, INPUT);

  // Initialize sensor
  if (!particleSensor.begin(Wire, I2C_SPEED_FAST)) //Use default I2C port, 400kHz speed
  {
    Serial.println("MAX30105 was not found. Please check wiring/power. ");
    while (1);
  }

  particleSensor.setup(); //Configure sensor with default settings
  particleSensor.setPulseAmplitudeRed(0x0A); //Turn Red LED to low to indicate sensor is running

  // Initialize valid HR reading array
  for (int i = 0; i < NUM_READINGS; i++) {
    valid_HR[i] = 0;
  }

}

void loop() {
  switch (currentState) {
    case INIT:
      if (digitalRead(BUTTONpin) == HIGH) {
        currentState = MEASURE;
        digitalWrite(LED1pin, HIGH);
        digitalWrite(LED2pin, HIGH);
        Serial.println("Pulse oximeter initialized.");
        Serial.println("When you're ready, place your index finger on the sensor with steady pressure and press the push button.");
      }
      delay(500);
      break;

    case MEASURE:
      digitalWrite(LED1pin, HIGH);
      digitalWrite(LED2pin, LOW);
      long irValue = particleSensor.getIR();

      if (display_ct == 100) {
        display_ct = 0; // display values only once in a while
      }

      if (checkForBeat(irValue) == true)
      {
        //We sensed a beat!
        long delta = millis() - lastBeat;
        lastBeat = millis();

        beatsPerMinute = 60 / (delta / 1000.0);

        if (beatsPerMinute < 220 && beatsPerMinute > 30)
        {
          rates[rateSpot++] = (byte)beatsPerMinute; //Store this reading in the array
          rateSpot %= RATE_SIZE; //Wrap variable

          //Take average of readings
          beatAvg = 0;
          for (byte x = 0 ; x < RATE_SIZE ; x++)
            beatAvg += rates[x];
          beatAvg /= RATE_SIZE;
        }
      }

      // Update the running average or non valid messages at regular intervals
      current_interval = millis() - lastUpdateMillis;
      noFinger_interval = millis() - lastnonValidMillis;


      if (irValue < 50000 && noFinger_interval >= 1000) {           // no valid reading
        lastnonValidMillis = millis();

        Serial.print(" No finger?");
        Serial.println();
      }
      else if (irValue >= 50000 && current_interval >= UPDATE_INTERVAL && beatAvg > 0) {  // sensing a presence, will determine if reading is valid

        lastUpdateMillis = millis();

        valid_HR[index_valid] = beatAvg;
        index_valid = (index_valid + 1) % NUM_READINGS;    // make sure you will only fill array until NUM_READINGS

        //Serial.println(current_interval);
       /* Serial.print("Valid BPMs are=");
        Serial.print(valid_HR[0] );
        Serial.print(valid_HR[1] );
        Serial.print(valid_HR[2] );
        Serial.print(valid_HR[3] );
        Serial.print(valid_HR[4] );
        Serial.print(valid_HR[5] );
        Serial.println();*/

        // Calculate the average and difference between max and min
        int sum = 0;
        int maxVal = valid_HR [0];
        int minVal = valid_HR [0];

        for (int i = 0; i < NUM_READINGS; i++) {
          sum += valid_HR[i];
          maxVal = max(valid_HR [i], maxVal);
          minVal = min(valid_HR [i], minVal);
        }
        int diffVal = maxVal - minVal;
        int avgHeartRate = sum / NUM_READINGS;

        if (diffVal <= 5) { // Adjust the threshold as needed
          Serial.print("Your current heart rate is ");
          Serial.print(avgHeartRate);
          Serial.print(" beats per minute.");
          currentState = INIT; // Transition to INIT state
          Serial.println("If you want to rescan, place your index finger on the sensor with steady pressure and press the push button.");
          // Clear the stored Heart Rate
          for (int i = 0; i < NUM_READINGS; i++) {
            valid_HR[i] = 0;
          }
        } else if (hasZeroElement()) {
          Serial.println("Initializing your reading");
        } else if (avgHeartRate <= 40) {
          Serial.println("You might not be making enough contact. Apply more pressure");
        } else {
          Serial.println("Calculating your heart rate...");
        }


      }
      break;
  }

}
